@@ -1,6 +1,6 @@
 /*  vcfutils.c -- allele-related utility functions.
 
-    Copyright (C) 2012-2014 Genome Research Ltd.
+    Copyright (C) 2012-2016 Genome Research Ltd.
 
     Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -22,7 +22,10 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.  */
 
+#include <config.h>
+
 #include "htslib/vcfutils.h"
+#include "htslib/kbitset.h"
 
 int bcf_calc_ac(const bcf_hdr_t *header, bcf1_t *line, int *ac, int which)
 {
@@ -61,10 +64,14 @@ int bcf_calc_ac(const bcf_hdr_t *header, bcf1_t *line, int *ac, int which)
                 case BCF_BT_INT8:  BRANCH_INT(int8_t); break;
                 case BCF_BT_INT16: BRANCH_INT(int16_t); break;
                 case BCF_BT_INT32: BRANCH_INT(int32_t); break;
-                default: fprintf(stderr, "[E::%s] todo: %d at %s:%d\n", __func__, ac_type, header->id[BCF_DT_CTG][line->rid].key, line->pos+1); exit(1); break;
+                default: hts_log_error("Unexpected type %d at %s:%d", ac_type, header->id[BCF_DT_CTG][line->rid].key, line->pos+1); exit(1); break;
             }
             #undef BRANCH_INT
-            assert( an>=nac );  // sanity check for missing values
+            if ( an<nac )
+            {
+                hts_log_error("Incorrect AN/AC counts at %s:%d", header->id[BCF_DT_CTG][line->rid].key, line->pos+1);
+                exit(1);
+            }
             ac[0] = an - nac;
             return 1;
         }
@@ -80,7 +87,7 @@ int bcf_calc_ac(const bcf_hdr_t *header, bcf1_t *line, int *ac, int which)
         for (i=0; i<(int)line->n_fmt; i++)
             if ( line->d.fmt[i].id==gt_id ) { fmt_gt = &line->d.fmt[i]; break; }
         if ( !fmt_gt ) return 0;
-        #define BRANCH_INT(type_t,missing,vector_end) { \
+        #define BRANCH_INT(type_t,vector_end) { \
             for (i=0; i<line->n_sample; i++) \
             { \
                 type_t *p = (type_t*) (fmt_gt->p + i*fmt_gt->size); \
@@ -88,16 +95,21 @@ int bcf_calc_ac(const bcf_hdr_t *header, bcf1_t *line, int *ac, int which)
                 for (ial=0; ial<fmt_gt->n; ial++) \
                 { \
                     if ( p[ial]==vector_end ) break; /* smaller ploidy */ \
-                    if ( !(p[ial]>>1) || p[ial]==missing ) continue; /* missing allele */ \
+                    if ( bcf_gt_is_missing(p[ial]) ) continue; /* missing allele */ \
+                    if ( p[ial]>>1 > line->n_allele ) \
+                    { \
+                        hts_log_error("Incorrect allele (\"%d\") in %s at %s:%d", (p[ial]>>1)-1, header->samples[i], header->id[BCF_DT_CTG][line->rid].key, line->pos+1); \
+                        exit(1); \
+                    } \
                     ac[(p[ial]>>1)-1]++; \
                 } \
             } \
         }
         switch (fmt_gt->type) {
-            case BCF_BT_INT8:  BRANCH_INT(int8_t,  bcf_int8_missing, bcf_int8_vector_end); break;
-            case BCF_BT_INT16: BRANCH_INT(int16_t, bcf_int16_missing, bcf_int16_vector_end); break;
-            case BCF_BT_INT32: BRANCH_INT(int32_t, bcf_int32_missing, bcf_int32_vector_end); break;
-            default: fprintf(stderr, "[E::%s] todo: %d at %s:%d\n", __func__, fmt_gt->type, header->id[BCF_DT_CTG][line->rid].key, line->pos+1); exit(1); break;
+            case BCF_BT_INT8:  BRANCH_INT(int8_t,  bcf_int8_vector_end); break;
+            case BCF_BT_INT16: BRANCH_INT(int16_t, bcf_int16_vector_end); break;
+            case BCF_BT_INT32: BRANCH_INT(int32_t, bcf_int32_vector_end); break;
+            default: hts_log_error("Unexpected type %d at %s:%d", fmt_gt->type, header->id[BCF_DT_CTG][line->rid].key, line->pos+1); exit(1); break;
         }
         #undef BRANCH_INT
         return 1;
@@ -108,12 +120,12 @@ int bcf_calc_ac(const bcf_hdr_t *header, bcf1_t *line, int *ac, int which)
 int bcf_gt_type(bcf_fmt_t *fmt_ptr, int isample, int *_ial, int *_jal)
 {
     int i, nals = 0, has_ref = 0, has_alt = 0, ial = 0, jal = 0;
-    #define BRANCH_INT(type_t,missing,vector_end) { \
+    #define BRANCH_INT(type_t,vector_end) { \
         type_t *p = (type_t*) (fmt_ptr->p + isample*fmt_ptr->size); \
         for (i=0; i<fmt_ptr->n; i++) \
         { \
             if ( p[i] == vector_end ) break; /* smaller ploidy */ \
-            if ( !p[i] || p[i] == missing ) continue; /* missing allele */ \
+            if ( bcf_gt_is_missing(p[i]) ) return GT_UNKN; /* missing allele */ \
             int tmp = p[i]>>1; \
             if ( tmp>1 ) \
             { \
@@ -137,10 +149,10 @@ int bcf_gt_type(bcf_fmt_t *fmt_ptr, int isample, int *_ial, int *_jal)
         } \
     }
     switch (fmt_ptr->type) {
-        case BCF_BT_INT8:  BRANCH_INT(int8_t,  bcf_int8_missing, bcf_int8_vector_end); break;
-        case BCF_BT_INT16: BRANCH_INT(int16_t, bcf_int16_missing, bcf_int16_vector_end); break;
-        case BCF_BT_INT32: BRANCH_INT(int32_t, bcf_int32_missing, bcf_int32_vector_end); break;
-        default: fprintf(stderr, "[E::%s] todo: fmt_type %d\n", __func__, fmt_ptr->type); exit(1); break;
+        case BCF_BT_INT8:  BRANCH_INT(int8_t,  bcf_int8_vector_end); break;
+        case BCF_BT_INT16: BRANCH_INT(int16_t, bcf_int16_vector_end); break;
+        case BCF_BT_INT32: BRANCH_INT(int32_t, bcf_int32_vector_end); break;
+        default: hts_log_error("Unexpected type %d", fmt_ptr->type); exit(1); break;
     }
     #undef BRANCH_INT
 
@@ -158,14 +170,15 @@ int bcf_gt_type(bcf_fmt_t *fmt_ptr, int isample, int *_ial, int *_jal)
 
 int bcf_trim_alleles(const bcf_hdr_t *header, bcf1_t *line)
 {
-    int i;
+    int i, ret = 0, nrm = 0;
+    kbitset_t *rm_set = NULL;
     bcf_fmt_t *gt = bcf_get_fmt(header, line, "GT");
     if ( !gt ) return 0;
 
     int *ac = (int*) calloc(line->n_allele,sizeof(int));
 
     // check if all alleles are populated
-    #define BRANCH(type_t,missing,vector_end) { \
+    #define BRANCH(type_t,vector_end) { \
         for (i=0; i<line->n_sample; i++) \
         { \
             type_t *p = (type_t*) (gt->p + i*gt->size); \
@@ -173,34 +186,57 @@ int bcf_trim_alleles(const bcf_hdr_t *header, bcf1_t *line)
             for (ial=0; ial<gt->n; ial++) \
             { \
                 if ( p[ial]==vector_end ) break; /* smaller ploidy */ \
-                if ( !(p[ial]>>1) || p[ial]==missing ) continue; /* missing allele */ \
-                if ( (p[ial]>>1)-1 >= line->n_allele ) return -1; \
+                if ( bcf_gt_is_missing(p[ial]) ) continue; /* missing allele */ \
+                if ( (p[ial]>>1)-1 >= line->n_allele ) { \
+                    hts_log_error("Allele index is out of bounds at %s:%d", header->id[BCF_DT_CTG][line->rid].key, line->pos+1); \
+                    ret = -1; \
+                    goto clean; \
+                } \
                 ac[(p[ial]>>1)-1]++; \
             } \
         } \
     }
     switch (gt->type) {
-        case BCF_BT_INT8:  BRANCH(int8_t,  bcf_int8_missing, bcf_int8_vector_end); break;
-        case BCF_BT_INT16: BRANCH(int16_t, bcf_int16_missing, bcf_int16_vector_end); break;
-        case BCF_BT_INT32: BRANCH(int32_t, bcf_int32_missing, bcf_int32_vector_end); break;
-        default: fprintf(stderr, "[E::%s] todo: %d at %s:%d\n", __func__, gt->type, header->id[BCF_DT_CTG][line->rid].key, line->pos+1); exit(1); break;
+        case BCF_BT_INT8:  BRANCH(int8_t,  bcf_int8_vector_end); break;
+        case BCF_BT_INT16: BRANCH(int16_t, bcf_int16_vector_end); break;
+        case BCF_BT_INT32: BRANCH(int32_t, bcf_int32_vector_end); break;
+        default: hts_log_error("Unexpected GT %d at %s:%d",
+            gt->type, header->id[BCF_DT_CTG][line->rid].key, line->pos + 1);
+            goto clean; break;
     }
     #undef BRANCH
 
-    int rm_als = 0, nrm = 0;
-    for (i=1; i<line->n_allele; i++)
-    {
-        if ( !ac[i] ) { rm_als |= 1<<i; nrm++; }
+    rm_set = kbs_init(line->n_allele);
+    for (i=1; i<line->n_allele; i++) {
+        if ( !ac[i] ) { kbs_insert(rm_set, i); nrm++; }
     }
-    free(ac);
 
-    if ( nrm ) bcf_remove_alleles(header, line, rm_als);
-    return nrm;
+    if (nrm) {
+        if (bcf_remove_allele_set(header, line, rm_set))
+            ret = -2;
+    }
+
+clean:
+    free(ac);
+    if (rm_set) kbs_destroy(rm_set);
+    return ret ? ret : nrm;
 }
 
 void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
+{
+    int i;
+    kbitset_t *rm_set = kbs_init(line->n_allele);
+    for (i=1; i<line->n_allele; i++)
+        if ( rm_mask & 1<<i ) kbs_insert(rm_set, i);
+
+    bcf_remove_allele_set(header, line, rm_set);
+    kbs_destroy(rm_set);
+}
+
+int bcf_remove_allele_set(const bcf_hdr_t *header, bcf1_t *line, const struct kbitset_t *rm_set)
 {
     int *map = (int*) calloc(line->n_allele, sizeof(int));
+    uint8_t *dat = NULL;
 
     // create map of indexes from old to new ALT numbering and modify ALT
     kstring_t str = {0,0,0};
@@ -209,7 +245,7 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
     int nrm = 0, i,j;  // i: ori alleles, j: new alleles
     for (i=1, j=1; i<line->n_allele; i++)
     {
-        if ( rm_mask & 1<<i )
+        if ( kbs_exists(rm_set, i) )
         {
             // remove this allele
             line->d.allele[i] = NULL;
@@ -221,11 +257,16 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
         map[i] = j;
         j++;
     }
-    if ( !nrm ) { free(map); free(str.s); return; }
+    if ( !nrm ) goto clean;
 
     int nR_ori = line->n_allele;
     int nR_new = line->n_allele-nrm;
-    assert(nR_new > 0); // should not be able to remove reference allele
+    if ( nR_new<=0 ) // should not be able to remove reference allele
+    {
+        hts_log_error("Cannot remove reference allele at %s:%d [%d]",
+            bcf_seqname(header,line), line->pos+1, nR_new);
+        goto err;
+    }
     int nA_ori = nR_ori-1;
     int nA_new = nR_new-1;
 
@@ -235,7 +276,6 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
     bcf_update_alleles_str(header, line, str.s);
 
     // remove from Number=G, Number=R and Number=A INFO fields.
-    uint8_t *dat = NULL;
     int mdat = 0, ndat = 0, mdat_bytes = 0, nret;
     for (i=0; i<line->n_info; i++)
     {
@@ -254,14 +294,16 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
         mdat_bytes = mdat * size;
         if ( nret<0 )
         {
-            fprintf(stderr,"[%s:%d %s] Could not access INFO/%s at %s:%d [%d]\n", __FILE__,__LINE__,__FUNCTION__,
+            hts_log_error("Could not access INFO/%s at %s:%d [%d]",
                 bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nret);
-            exit(1);
+            goto err;
         }
+        if ( nret==0 ) continue; // no data for this tag
+
         if ( type==BCF_HT_STR )
         {
             str.l = 0;
-            char *ss = (char*) dat, *se = (char*) dat;
+            char *ss = (char*) dat, *se = (char*) dat, s = ss[0];
             if ( vlen==BCF_VL_A || vlen==BCF_VL_R )
             {
                 int nexp, inc = 0;
@@ -276,7 +318,7 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                 {
                     if ( !*se ) break;
                     while ( *se && *se!=',' ) se++;
-                    if ( rm_mask & 1<<(j+inc) )
+                    if ( kbs_exists(rm_set, j+inc) )
                     {
                         if ( *se ) se++;
                         ss = se;
@@ -287,7 +329,13 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                     if ( *se ) se++;
                     ss = se;
                 }
-                assert( j==nexp );
+                if ( j==1 && s == '.' ) continue; // missing
+                if ( j!=nexp )
+                {
+                    hts_log_error("Unexpected number of values in INFO/%s at %s:%d; expected Number=%c=%d, but found %d",
+                        bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, vlen==BCF_VL_A ? 'A' : 'R', nexp, j);
+                    goto err;
+                }
             }
             else    // Number=G, assuming diploid genotype
             {
@@ -299,7 +347,7 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                         if ( !*se ) break;
                         while ( *se && *se!=',' ) se++;
                         n++;
-                        if ( rm_mask & 1<<j || rm_mask & 1<<k )
+                        if ( kbs_exists(rm_set, j) || kbs_exists(rm_set, k) )
                         {
                             if ( *se ) se++;
                             ss = se;
@@ -312,32 +360,66 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                     }
                     if ( !*se ) break;
                 }
-                assert( n=nG_ori );
+                if ( n==1 && s == '.' ) continue; // missing
+                if ( n!=nG_ori )
+                {
+                    hts_log_error("Unexpected number of values in INFO/%s at %s:%d; expected Number=G=%d, but found %d",
+                        bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nG_ori, n);
+                    goto err;
+                }
             }
 
             nret = bcf_update_info(header, line, bcf_hdr_int2id(header,BCF_DT_ID,info->key), (void*)str.s, str.l, type);
             if ( nret<0 )
             {
-                fprintf(stderr,"[%s:%d %s] Could not update INFO/%s at %s:%d [%d]\n", __FILE__,__LINE__,__FUNCTION__,
-                        bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nret);
-                exit(1);
+                hts_log_error("Could not update INFO/%s at %s:%d [%d]",
+                    bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nret);
+                goto err;
             }
             continue;
         }
 
+        if (nret==1) // could be missing - check
+        {
+            int missing = 0;
+            #define BRANCH(type_t, is_missing) { \
+                type_t *p = (type_t *) info->vptr; \
+                if ( is_missing ) missing = 1; \
+            }
+            switch (info->type) {
+                case BCF_BT_INT8:  BRANCH(int8_t,  p[0]==bcf_int8_missing); break;
+                case BCF_BT_INT16: BRANCH(int16_t, p[0]==bcf_int16_missing); break;
+                case BCF_BT_INT32: BRANCH(int32_t, p[0]==bcf_int32_missing); break;
+                case BCF_BT_FLOAT: BRANCH(float,   bcf_float_is_missing(p[0])); break;
+                default: hts_log_error("Unexpected type %d", info->type); goto err; break;
+            }
+            #undef BRANCH
+            if (missing) continue; // could remove this INFO tag?
+        }
+
         if ( vlen==BCF_VL_A || vlen==BCF_VL_R )
         {
             int inc = 0, ntop;
             if ( vlen==BCF_VL_A )
             {
-                assert( nret==nA_ori );
+                if ( nret!=nA_ori )
+                {
+                    hts_log_error("Unexpected number of values in INFO/%s at %s:%d; expected Number=A=%d, but found %d",
+                        bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nA_ori, nret);
+                    goto err;
+                }
                 ntop = nA_ori;
                 ndat = nA_new;
                 inc  = 1;
             }
             else
             {
-                assert( nret==nR_ori );
+                if ( nret!=nR_ori )
+                {
+                    hts_log_error("Unexpected number of values in INFO/%s at %s:%d; expected Number=R=%d, but found %d",
+                        bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nR_ori, nret);
+                    goto err;
+                }
                 ntop = nR_ori;
                 ndat = nR_new;
             }
@@ -350,7 +432,7 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                 for (j=0; j<ntop; j++) /* j:ori, k:new */ \
                 { \
                     if ( is_vector_end ) { memcpy(dat+k*size, dat+j*size, size); break; } \
-                    if ( rm_mask & 1<<(j+inc) ) continue; \
+                    if ( kbs_exists(rm_set, j+inc) ) continue; \
                     if ( j!=k ) memcpy(dat+k*size, dat+j*size, size); \
                     k++; \
                 } \
@@ -364,7 +446,12 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
         }
         else    // Number=G
         {
-            assert( nret==nG_ori );
+            if ( nret!=nG_ori )
+            {
+                hts_log_error("Unexpected number of values in INFO/%s at %s:%d; expected Number=R=%d, but found %d",
+                    bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nG_ori, nret);
+                goto err;
+            }
             int k, l_ori = -1, l_new = 0;
             ndat = nG_new;
 
@@ -378,7 +465,7 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                     { \
                         l_ori++; \
                         if ( is_vector_end ) { memcpy(dat+l_new*size, dat+l_ori*size, size); break; } \
-                        if ( rm_mask & 1<<j || rm_mask & 1<<k ) continue; \
+                        if ( kbs_exists(rm_set, j) || kbs_exists(rm_set, k) ) continue; \
                         if ( l_ori!=l_new ) memcpy(dat+l_new*size, dat+l_ori*size, size); \
                         l_new++; \
                     } \
@@ -395,9 +482,9 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
         nret = bcf_update_info(header, line, bcf_hdr_int2id(header,BCF_DT_ID,info->key), (void*)dat, ndat, type);
         if ( nret<0 )
         {
-            fprintf(stderr,"[%s:%d %s] Could not update INFO/%s at %s:%d [%d]\n", __FILE__,__LINE__,__FUNCTION__,
-                    bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nret);
-            exit(1);
+            hts_log_error("Could not update INFO/%s at %s:%d [%d]",
+                bcf_hdr_int2id(header,BCF_DT_ID,info->key), bcf_seqname(header,line), line->pos+1, nret);
+            goto err;
         }
     }
 
@@ -416,15 +503,26 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
             {
                 for (j=0; j<nret; j++)
                 {
-                    if ( ptr[j]==bcf_gt_missing ) continue;
+                    if ( bcf_gt_is_missing(ptr[j]) ) continue;
                     if ( ptr[j]==bcf_int32_vector_end ) break;
                     int al = bcf_gt_allele(ptr[j]);
-                    assert( al<nR_ori && map[al]>=0 );
+                    if ( !( al<nR_ori && map[al]>=0 ) )
+                    {
+                        hts_log_error("Problem updating genotypes at %s:%d [ al<nR_ori && map[al]>=0 :: al=%d,nR_ori=%d,map[al]=%d ]",
+                            bcf_seqname(header,line), line->pos+1, al, nR_ori, map[al]);
+                        goto err;
+                    }
                     ptr[j] = (map[al]+1)<<1 | (ptr[j]&1);
                 }
                 ptr += nret;
             }
-            bcf_update_genotypes(header, line, (void*)dat, nret*line->n_sample);
+            nret = bcf_update_genotypes(header, line, (void*)dat, nret*line->n_sample);
+            if ( nret<0 )
+            {
+                hts_log_error("Could not update FORMAT/GT at %s:%d [%d]",
+                    bcf_seqname(header,line), line->pos+1, nret);
+                goto err;
+            }
         }
     }
 
@@ -448,10 +546,11 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
         mdat_bytes = mdat * size;
         if ( nret<0 )
         {
-            fprintf(stderr,"[%s:%d %s] Could not access FORMAT/%s at %s:%d [%d]\n", __FILE__,__LINE__,__FUNCTION__,
-                    bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nret);
-            exit(1);
+            hts_log_error("Could not access FORMAT/%s at %s:%d [%d]",
+                bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nret);
+            goto err;
         }
+        if ( nret == 0 ) continue; // no data for this tag
 
         if ( type==BCF_HT_STR )
         {
@@ -469,13 +568,13 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                     nexp = nR_ori;
                 for (j=0; j<line->n_sample; j++)
                 {
-                    char *ss = ((char*)dat) + j*size, *se = ss + size, *ptr = ss;
+                    char *ss = ((char*)dat) + j*size, *se = ss + size, *ptr = ss, s = ss[0];
                     int k_src = 0, k_dst = 0, l = str.l;
                     for (k_src=0; k_src<nexp; k_src++)
                     {
                         if ( ptr>=se || !*ptr) break;
                         while ( ptr<se && *ptr && *ptr!=',' ) ptr++;
-                        if ( rm_mask & 1<<(k_src+inc) )
+                        if ( kbs_exists(rm_set, k_src+inc) )
                         {
                             ss = ++ptr;
                             continue;
@@ -485,7 +584,13 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                         ss = ++ptr;
                         k_dst++;
                     }
-                    assert( k_src==nexp );
+                    if ( k_src==1 && s == '.' ) continue; // missing
+                    if ( k_src!=nexp )
+                    {
+                        hts_log_error("Unexpected number of values in FORMAT/%s at %s:%d; expected Number=%c=%d, but found %d",
+                            bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, vlen==BCF_VL_A ? 'A' : 'R', nexp, k_src);
+                        goto err;
+                    }
                     l = str.l - l;
                     for (; l<size; l++) kputc(0, &str);
                 }
@@ -494,7 +599,7 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
             {
                 for (j=0; j<line->n_sample; j++)
                 {
-                    char *ss = ((char*)dat) + j*size, *se = ss + size, *ptr = ss;
+                    char *ss = ((char*)dat) + j*size, *se = ss + size, *ptr = ss, s = ss[0];
                     int k_src = 0, k_dst = 0, l = str.l;
                     int nexp = 0; // diploid or haploid?
                     while ( ptr<se )
@@ -504,7 +609,13 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                         ptr++;
                     }
                     if ( ptr!=ss ) nexp++;
-                    assert( nexp==nG_ori || nexp==nR_ori );
+                    if ( nexp==1 && s == '.' ) continue; // missing
+                    if ( nexp!=nG_ori && nexp!=nR_ori )
+                    {
+                        hts_log_error("Unexpected number of values in FORMAT/%s at %s:%d; expected Number=G=%d(diploid) or %d(haploid), but found %d",
+                            bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nG_ori, nR_ori, nexp);
+                        goto err;
+                    }
                     ptr = ss;
                     if ( nexp==nG_ori ) // diploid
                     {
@@ -515,7 +626,7 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                             {
                                 if ( ptr>=se || !*ptr ) break;
                                 while ( ptr<se && *ptr && *ptr!=',' ) ptr++;
-                                if ( rm_mask & 1<<ia || rm_mask & 1<<ib )
+                                if ( kbs_exists(rm_set, ia) || kbs_exists(rm_set, ib) )
                                 {
                                     ss = ++ptr;
                                     continue;
@@ -534,7 +645,7 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                         {
                             if ( ptr>=se || !*ptr ) break;
                             while ( ptr<se && *ptr && *ptr!=',' ) ptr++;
-                            if ( rm_mask & 1<<k_src )
+                            if ( kbs_exists(rm_set, k_src) )
                             {
                                 ss = ++ptr;
                                 continue;
@@ -544,7 +655,12 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                             ss = ++ptr;
                             k_dst++;
                         }
-                        assert( k_src==nR_ori );
+                        if ( k_src!=nR_ori )
+                        {
+                            hts_log_error("Unexpected number of values in FORMAT/%s at %s:%d; expected Number=G=%d(haploid), but found %d",
+                                bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nR_ori, k_src);
+                            goto err;
+                        }
                         l = str.l - l;
                         for (; l<size; l++) kputc(0, &str);
                     }
@@ -553,29 +669,60 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
             nret = bcf_update_format(header, line, bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), (void*)str.s, str.l, type);
             if ( nret<0 )
             {
-                fprintf(stderr,"[%s:%d %s] Could not update FORMAT/%s at %s:%d [%d]\n", __FILE__,__LINE__,__FUNCTION__,
-                        bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nret);
-                exit(1);
+                hts_log_error("Could not update FORMAT/%s at %s:%d [%d]",
+                    bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nret);
+                goto err;
             }
             continue;
         }
 
         int nori = nret / line->n_sample;
+        if ( nori==1 && !(vlen==BCF_VL_A && nori==nA_ori) ) // all values may be missing - check
+        {
+            int all_missing = 1;
+            #define BRANCH(type_t, is_missing) { \
+                for (j=0; j<line->n_sample; j++) \
+                { \
+                    type_t *p = (type_t*) (fmt->p + j*fmt->size); \
+                    if ( !(is_missing)) { all_missing = 0; break; } \
+                } \
+            }
+            switch (fmt->type) {
+                case BCF_BT_INT8:  BRANCH(int8_t,  p[0]==bcf_int8_missing); break;
+                case BCF_BT_INT16: BRANCH(int16_t, p[0]==bcf_int16_missing); break;
+                case BCF_BT_INT32: BRANCH(int32_t, p[0]==bcf_int32_missing); break;
+                case BCF_BT_FLOAT: BRANCH(float,   bcf_float_is_missing(p[0])); break;
+                default: hts_log_error("Unexpected type %d", fmt->type); goto err; break;
+            }
+            #undef BRANCH
+            if (all_missing) continue; // could remove this FORMAT tag?
+        }
+
         if ( vlen==BCF_VL_A || vlen==BCF_VL_R || (vlen==BCF_VL_G && nori==nR_ori) ) // Number=A, R or haploid Number=G
         {
-            int ntop, inc = 0;
+            int inc = 0, nnew;
             if ( vlen==BCF_VL_A )
             {
-                assert( nori==nA_ori );     // todo: will fail if all values are missing
-                ntop = nA_ori;
+                if ( nori!=nA_ori )
+                {
+                    hts_log_error("Unexpected number of values in FORMAT/%s at %s:%d; expected Number=A=%d, but found %d",
+                        bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nA_ori, nori);
+                    goto err;
+                }
                 ndat = nA_new*line->n_sample;
+                nnew = nA_new;
                 inc  = 1;
             }
             else
             {
-                assert( nori==nR_ori );     // todo: will fail if all values are missing
-                ntop = nR_ori;
+                if ( nori!=nR_ori )
+                {
+                    hts_log_error("Unexpected number of values in FORMAT/%s at %s:%d; expected Number=R=%d, but found %d",
+                        bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nR_ori, nori);
+                    goto err;
+                }
                 ndat = nR_new*line->n_sample;
+                nnew = nR_new;
             }
 
             #define BRANCH(type_t,is_vector_end) \
@@ -583,14 +730,14 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                 for (j=0; j<line->n_sample; j++) \
                 { \
                     type_t *ptr_src = ((type_t*)dat) + j*nori; \
-                    type_t *ptr_dst = ((type_t*)dat) + j*nA_new; \
+                    type_t *ptr_dst = ((type_t*)dat) + j*nnew; \
                     int size = sizeof(type_t); \
                     int k_src, k_dst = 0; \
-                    for (k_src=0; k_src<ntop; k_src++) \
+                    for (k_src=0; k_src<nori; k_src++) \
                     { \
                         if ( is_vector_end ) { memcpy(ptr_dst+k_dst, ptr_src+k_src, size); break; } \
-                        if ( rm_mask & 1<<(k_src+inc) ) continue; \
-                        if ( k_src!=k_dst ) memcpy(ptr_dst+k_dst, ptr_src+k_src, size); \
+                        if ( kbs_exists(rm_set, k_src+inc) ) continue; \
+                        memcpy(ptr_dst+k_dst, ptr_src+k_src, size); \
                         k_dst++; \
                     } \
                 } \
@@ -604,7 +751,12 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
         }
         else    // Number=G, diploid or mixture of haploid+diploid
         {
-            assert( nori==nG_ori );
+            if ( nori!=nG_ori )
+            {
+                hts_log_error("Unexpected number of values in FORMAT/%s at %s:%d; expected Number=G=%d, but found %d",
+                    bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nG_ori, nori);
+                goto err;
+            }
             ndat = nG_new*line->n_sample;
 
             #define BRANCH(type_t,is_vector_end) \
@@ -621,8 +773,8 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                     { \
                         for (k_src=0; k_src<nR_ori; k_src++) \
                         { \
-                            if ( rm_mask & 1<<k_src ) continue; \
-                            if ( k_src!=k_dst ) memcpy(ptr_dst+k_dst, ptr_src+k_src, size); \
+                            if ( kbs_exists(rm_set, k_src) ) continue; \
+                            memcpy(ptr_dst+k_dst, ptr_src+k_src, size); \
                             k_dst++; \
                         } \
                         memcpy(ptr_dst+k_dst, ptr_src+k_src, size); \
@@ -635,9 +787,9 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
                             for (ib=0; ib<=ia; ib++) \
                             { \
                                 k_src++; \
-                                if ( is_vector_end ) { memcpy(ptr_dst+k_dst, ptr_src+k_src, size); ia = nR_ori; break; } \
-                                if ( rm_mask & 1<<ia || rm_mask & 1<<ib ) continue; \
-                                if ( k_src!=k_dst ) memcpy(ptr_dst+k_dst, ptr_src+k_src, size); \
+                                if ( is_vector_end ) { memcpy(ptr_dst+k_dst, ptr_src+k_src, size); ia = nR_ori; break; }  \
+                                if ( kbs_exists(rm_set, ia) || kbs_exists(rm_set, ib) ) continue; \
+                                memcpy(ptr_dst+k_dst, ptr_src+k_src, size); \
                                 k_dst++; \
                             } \
                         } \
@@ -654,13 +806,22 @@ void bcf_remove_alleles(const bcf_hdr_t *header, bcf1_t *line, int rm_mask)
         nret = bcf_update_format(header, line, bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), (void*)dat, ndat, type);
         if ( nret<0 )
         {
-            fprintf(stderr,"[%s:%d %s] Could not update FORMAT/%s at %s:%d [%d]\n", __FILE__,__LINE__,__FUNCTION__,
-                    bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nret);
-            exit(1);
+            hts_log_error("Could not update FORMAT/%s at %s:%d [%d]",
+                bcf_hdr_int2id(header,BCF_DT_ID,fmt->id), bcf_seqname(header,line), line->pos+1, nret);
+            goto err;
         }
     }
-    free(dat);
+
+clean:
     free(str.s);
     free(map);
+    free(dat);
+    return 0;
+
+err:
+    free(str.s);
+    free(map);
+    free(dat);
+    return -1;
 }
 
